@@ -8,6 +8,44 @@ class Scorer:
     cmScaleByClass = 4
 
     metricNameMap = {}
+    addedMetrics = {}
+    
+    def getMetricDefinition(self, name):
+        
+        metricDefinition = {
+            # init value, useAsBest, biggerIsBetter
+            "AUC": [0.5, False, True],
+            "AP": [0.0, True, True],
+            "accuracy": [0.0, False, True],
+            "kappa": [-1, True, True],
+            "CSI": [0, True, True],
+            "loss": [np.inf, True, False],
+            "meanSquared": [np.inf, True, False],
+            "SNR": [np.inf, False, False],
+            "confusion": [np.inf, False, False],
+            "auroc": [0.0, False, True],
+            "auprc": [0.0, True, True],
+            "eventCountDiff": [np.inf, False, False],
+            "positiveErrorRatio": [np.inf, True, False],
+        }
+        
+        if name in Scorer.addedMetrics:
+            metric = Scorer.addedMetrics[name]
+            return [metric["initValue"], metric["useAsBest"], metric["biggerIsBetter"]]
+
+        return metricDefinition[name]
+        
+    
+    @staticmethod
+    def registerMetric(metricName, metricFunction, combineFunction, biggerIsBetter=True, useAsBest=True, initValue=None):
+        
+        Scorer.addedMetrics[metricName] = {
+            "score": metricFunction,
+            "combine": combineFunction,
+            "biggerIsBetter": biggerIsBetter,
+            "useAsBest": useAsBest,
+            "initValue": initValue if initValue is not None else 0.0 if biggerIsBetter else np.inf
+        }
 
     def getMetricName(self, metricName):
         if metricName in Scorer.metricNameMap:
@@ -77,7 +115,7 @@ class Scorer:
 
     def recordEntry(self, truth, prediction, metrics, recordName=None):
 
-        prediction = self.flattenPrediction(prediction)
+        # prediction = self.flattenPrediction(prediction)
 
         # if "confusion" in metrics and self.addEvents:
         #     events = self.getValidationEvents(truth, prediction, recordName=recordName)
@@ -109,6 +147,11 @@ class Scorer:
         metricName = self.getMetricName(metricName)
 
         result = None
+              
+        if metricName in self.addedMetrics:
+            truth = np.array([self.recordResult[id]["truth"] for id in self.recordResult])
+            prediction = np.array([self.recordResult[id]["prediction"] for id in self.recordResult])
+            self.results[metricName] = self.addedMetrics[metricName]["combine"](truth, prediction)
         if metricName in ["kappa", "accuracy"]:  # recalcuate from confusion
             self.combineMetric("confusion")
             self.scoreMetric("kappa", None, None)
@@ -150,6 +193,7 @@ class Scorer:
 
         if result is not None:
             self.results[metricName] = result
+            
 
         return self.results[metricName]
 
@@ -264,7 +308,10 @@ class Scorer:
             return self.results[metricName]
 
         result = None
-        if metricName in ["kappa", "accuracy"]:
+        
+        if metricName in self.addedMetrics:
+            self.results[metricName] = self.addedMetrics[metricName]["score"](truth, prediction)  
+        elif metricName in ["kappa", "accuracy"]:
             confusion = self.scoreMetric("confusion", truth, prediction)
             k, acc = self.calculateKappaAccFromConfusion(confusion)
             self.results["accuracy"] = acc
